# -*- coding: utf-8 -*-
"""
source.cms_pages.templatetags.settings_value description
"""
from django import template
from config import settings

register = template.Library()

# settings value
@register.simple_tag
def settings_value(name):
    allowed_keys = ('APP_VERSION',
                    'AVAILABLE_TXT',
                    'SUBMITTED_TXT',
                    'BASE_URL',
                    'LANGUAGES',
                    'TIME_ZONE',
                    'LOCAL_TIMEZONE',
                    'TERMS_OF_SERVICE',
                    )
    if name in allowed_keys:
        return getattr(settings, name, "")
    
    return ''



# -*- coding: utf-8 -*-
import re
import os
import logging
import xml.dom.minidom

from pyxform.xls2xform import xls2xform_convert

from hashlib import md5
from datetime import datetime

from django.db import models
from django.utils.translation import gettext as _
from django.urls import reverse
from django.contrib.auth import get_user_model

Profile = get_user_model()

__author__ = 'jean pierre huart, patrick houben'
__email__ = 'j.huart@cra.wallonie.be, p.houben@cra.wallonie.be'
__copyright__ = 'Copyright 2021, Jean Pierre Huart, Patrick Houben'
__license__ = 'GPLv3'
__date__ = '2021-03-05'
__version__ = '0.1 dev'
__status__ = 'Development'


LOG = logging.getLogger(__name__)
LOG_DEBUG = logging.getLogger('mydebug')


def xform_path(instance, filename):
    """
    Define where to save posted xforms
    """
    my_xform_path = os.path.join(
        instance.class_name,
        os.path.split(filename)[1]
    )
    return my_xform_path


def get_form_id(xml_content):
    """
    Search id attr in data tag
    """
    my_regexp = re.compile(r'<data.*id="([^"]+)"', re.DOTALL)
    matches = my_regexp.findall(xml_content)
    if matches:
        return matches[0]
    else:
        LOG.error(f"Unable to find 'id attr' of <data> tag in xml file")
        return 'ERROR'


def get_version(xml_content):
    """
    Search version attr in data tag in XML and set it in table
    """
    my_regexp = re.compile(r'<data.*version="([^"]+)"', re.DOTALL)
    matches = my_regexp.findall(xml_content)
    if matches:
        return matches[0]
    else:
        # Version might not be present in settings tab or tab is missing!
        return get_form_id(xml_content)


class XForm(models.Model):
    """
    XForm management
    necessary steps through admin
    1. Build an XLSForm file https://xlsform.org/en/
    2. Submit this file on https://getodk.org/xlsform/
    3. Check file correctness while clicking on 'Preview in browser' button
    4. Copy-paste XML content generated in xml_content field
    => The file is ready to read in ODK Collect app
    """
    xls_file = models.FileField(
        verbose_name=_("Excel file"),
        upload_to=xform_path,
        blank=True, null=True,
        help_text=_("XLSForm with 3 tabs: survey, choices, settings")
    )
    xml_file = models.FileField(
        verbose_name=_("XML file"),
        upload_to=xform_path,
        blank=True, null=True,
        help_text="XLSForm "+_("converted by")+" <a href='https://getodk.org/xlsform/' target='_blank'>https://getodk.org/xlsform/</a>"
    )
    xml_content = models.TextField(
        verbose_name=_("Content of XML form")
    )
    form_id = models.SlugField(
        editable=False,
        max_length=200,
        help_text=_("Retrieved from XLSForm (xls settings tab)")
    )
    version = models.CharField(
        editable=False,
        max_length=200,
        help_text=_("Retrieved from XLSForm (xls settings tab)")
    )
    title = models.CharField(
        verbose_name=_("Title"),
        editable=False, max_length=250,
        help_text=_("Retrieved from XLSForm (xls settings tab)")
    )
    short_desc = models.CharField(
        max_length=250,
        verbose_name=_("Short description"),
        blank=True
    )
    created_by = models.ForeignKey(
        Profile,
        related_name="xform_created_by",
        on_delete=models.CASCADE, verbose_name=_("Created by")
    )
    created_on = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_("Created on")
    )
    model_created_on = models.DateTimeField(
        verbose_name=_("Model created on"),
        null=True, blank=True
    )

    class Meta:
        verbose_name = _("Available form")
        unique_together = ('form_id', 'version')
        ordering = ('xml_file',)

    @property
    def class_name(self):
        return self.__class__.__name__

    def _set_creator(self, current_user):
        if not self.created_by:
            self.created_by = current_user

    def _set_xml_content(self):
        try:
            with open(self.xml_file.path, 'r') as f:
                # myxml = xml.dom.minidom.parseString(f.read())
                # self.xml_content = myxml.toprettyxml()
                self.xml_content = f.read()
        except Exception as xcpt:
            raise Exception(f"XML file not found in {self.xml_file.path}")
        return

    def _set_form_id(self):
        """
        Search over <instance> tag in xml content and store id
        """
        my_regexp = re.compile(r'<instance>.*?id="([^"]+)".*</instance>',
                               re.DOTALL)
        match = my_regexp.findall(self.xml_content)
        if match:
            self.form_id = match[0]
            return True
        else:
            return False

    def _set_version(self):
        """
        Search over <instance> tag in xml content and store version
        """
        my_regexp = re.compile(r'<instance>.*?version="([^"]+)".*</instance>',
                               re.DOTALL)
        matches = my_regexp.findall(self.xml_content)
        if matches:
            self.version = matches[0]
        else:
            self.version = self.form_id
        return

    def _set_title(self):
        """
        Search over <h:title> tag in xml content and store form title
        \s+ => any white space character
        ^ => beginning
        $ => end
        """
        text = re.sub(r'\s+', ' ', self.xml_content)
        matches = re.findall(r'<h:title>([^<]+)</h:title>', text)
        if len(matches) != 1:
            raise Exception('There should be a single title.', matches)
        self.title = '' if not matches else matches[0]
        return

    def process_xform(self):
        xlspath = self.xls_file.path
        xmlpath = xlspath.replace(".xlsx", ".xml")
        response = xls2xform_convert(xlspath, xmlpath)
        # TODO: to be improved!
        if response:
            raise Exception(response)
        else:
            self.xml_file = self.xls_file.name.replace(".xlsx", ".xml")
            self._set_xml_content()
            self._set_form_id()
            self._set_version()
            self._set_title()
        return

    def get_absolute_url(self):
        return reverse('odk:xform_detail', args=[str(self.id)])

    def get_title(self):
        return self._meta.verbose_name

    def __str__(self):
        filename = str(self.xls_file).replace(f"{self.class_name}/", '')
        form_id = getattr(self, 'form_id', '')
        version = getattr(self, 'version', '')
        form_version = f"{form_id} {version}"
        return filename

    @property
    def hash(self):
        return u'md5:%s' % md5(self.xml_content.encode('utf8')).hexdigest()


#############################################################

def submission_path(instance, filename):
    """
    Define where to save posted xforms
    """
    path = os.path.join(
        instance.class_name,
        instance.subfolder1,
        instance.subfolder2,
        os.path.split(filename)[1]
    )
    return path


def get_survey_date(xml_content):
    matches = re.findall(r'<today>([^<]+)</today>', xml_content)
    if matches:
        return matches[0]
    else:
        return None


def get_instanceid(xml_content):
    """
    unique identifier per survey
    """
    matches = re.findall(r'<instanceID>uuid:([^<]+)</instanceID>', xml_content)
    if matches:
        return matches[0]
    else:
        LOG.error("Unable to find '<instanceID>uuid:' pattern in xml file")
        return None


class XFormSubmit(models.Model):
    """
    Submitted files from ODK Collect
    """
    # xform = models.ForeignKey(
    #     XForm,
    #     on_delete=models.PROTECT,
    #     verbose_name="Template form",
    #     related_name="template_form"
    # )
    form_id = models.SlugField(
        editable=False,
        max_length=200,
        help_text=_("Retrieved from XML form (settings tab of Excel file)")
    )
    version = models.CharField(
        editable=False,
        max_length=200,
        help_text=_("Retrieved from XML form (settings tab of Excel file)")
    )
    instanceid = models.UUIDField(unique=True)
    deviceid = models.CharField(
        max_length=255,
        blank=True, null=True
    )
    survey_date = models.DateField(
        verbose_name=_("Encoding date"),
        blank=True, null=True
    )
    picture_files = models.JSONField(
        verbose_name=_("Picture file names"),
        blank=True, null=True
    )
    xml_file = models.FileField(
        verbose_name=_('Submitted form'),
        help_text=_("XML file sended through ODK Collect Mobile App"),
        blank=True
    )
    xml_content = models.TextField(
        verbose_name=_("Content of XML file submitted"),
        help_text=_("XML content sended through ODK Collect Mobile App")
    )
    submitted_by = models.CharField(
        max_length=255,
        verbose_name=_("Submitted by"),
        blank=True, null=True
    )
    submitted_on = models.DateTimeField(
        verbose_name=_("Submitted on")
    )
    inserted_on = models.DateTimeField(
        verbose_name=_("Inserted on"),
        blank=True, null=True
    )    

    class Meta:
        verbose_name = _("Submitted form")
        # unique_together = ('form_id', 'version', 'instanceid')
        ordering = ('submitted_on',)

    @property
    def class_name(self):
        return self.__class__.__name__

    @property
    def subfolder1(self):
        """folder name: form_id-version"""
        return f"{self.form_id}-{self.version}"

    @property
    def subfolder2(self):
        """survey date in format YYYY-MM-DD"""
        return self.survey_date.strftime('%Y-%m-%d')

    def _set_xml_content(self):
        with open(self.xml_file.path, 'r') as f:
            myxml = xml.dom.minidom.parseString(f.read())
            self.xml_content = myxml.toprettyxml()        
        # x = etree.parse(self.xml_file.path)
        # self.xml_content = etree.tostring(x, pretty_print=True, encoding=str)
        return

    def _set_form_id(self):
        self.form_id = get_form_id(self.xml_content)

    def _set_version(self):
        self.version = get_version(self.xml_content)

    def _set_survey_date(self):
        self.survey_date = get_survey_date(self.xml_content)
        if self.survey_date is None:
            self.survey_date = datetime.now().strftime("%Y-%m-%d")

    def _set_username(self):
        matches = re.findall(r'<username>([^<]+)</username>', self.xml_content)
        if matches:
            self.submitted_by = matches[0]
            return
        else:
            LOG.warning("Unable to find '<username>' pattern in xml file")
            return

    def _set_deviceid(self):
        matches = re.findall(r'<deviceid>([^<]+)</deviceid>', self.xml_content)
        if matches:
            self.deviceid = matches[0]
            return
        else:
            LOG.warning("Unable to find '<deviceid>' pattern in xml file")
            return

    def _set_instanceid(self):
        self.instanceid = get_instanceid(self.xml_content)

    def rm_xmlext_from_url(self):
        """
        Get pictures in the appropriate folder, ie:
        pic_path = os.path.join(
                        "XFormSubmit",
                        subfolder1 (form_id-version),
                        subfolder2 (date as yyyy-mm-dd),
                        xml_filename,
                        pic_pointer.name
                    )
        """
        url_path = self.xml_file.url.replace('.xml', '/')
        return url_path

    def save(self, *args, **kwargs):
        """
        Save form_id, instanceid contained in xml file
        """
        self._set_xml_content()
        self._set_form_id()
        self._set_version()
        self._set_survey_date()
        self._set_username()
        # self._set_deviceid()
        old_instanceid = self.instanceid
        self._set_instanceid()
        # check if we have an existing instanceid,
        # if so, the one must match but only if xformsubmit is NOT new
        if self.pk and old_instanceid and old_instanceid.__str__() != self.instanceid:
            raise Exception(
                f"Your updated form's instanceid '{self.instanceid}' must match the existing forms' instanceid '{old_instanceid}'."
                  )

        super(XFormSubmit, self).save(*args, **kwargs)
        return

    def get_absolute_url(self):
        return reverse('odk:xformsubmit_detail', args=[str(self.id)])

    def get_title(self):
        return self._meta.verbose_name

    def __str__(self):
        return self.xml_file.name



# -*- coding: utf-8 -*-
from django.http import Http404
from django.shortcuts import render
from django.utils import timezone

from config.settings import USE_HTTPS
from odk.models import XForm
from odk.openrosa.utils_push import response_with_mimetype_and_name



def rosa_list(request):
    """
    This is where ODK Collect gets its download list.
    xformsList format can be found https://docs.getodk.org/openrosa-form-list/
    """
    # Get all the forms in the DB
    xforms = XForm.objects.all()

    # Prepare context to send back
    PROTOCOL = 'https' if USE_HTTPS else 'http'
    context = {
        'host': f"{PROTOCOL}://{request.get_host()}",
        'object_list': xforms,
    }

    # Sending back the datas with the xformslist format
    response = render(
        request,
        "xformsList.xml",
        context,
        content_type="text/xml; charset=utf-8"
    )
    response['X-OpenRosa-Version'] = '1.0'
    response['Date'] = timezone.now().strftime('%a, %d %b %Y %H:%M:%S %Z')
    return response


def rosa_detail(request, form_id):
    """
    Download XForm XML view.
    """
    try:
        obj = XForm.objects.get(form_id=form_id)
    except XForm.DoesNotExist:
        return Http404("XForm does not exist.")

    response = response_with_mimetype_and_name('xml', form_id, show_date=False)
    response.content = obj.xml_content
    return response



# -*- coding: utf-8 -*-
# utils for xml_content management in XForm & XFormSubmit
import os
import re
import logging

from datetime import datetime
# Django
from django.utils.translation import gettext as _

# App
from odk.storage import overwrite_storage
from odk.models import (
    XForm, XFormSubmit,
    get_form_id,
    get_version,
    get_survey_date,
    get_instanceid
)

LOG = logging.getLogger(__name__)
LOG_DEBUG = logging.getLogger('mydebug')


def get_submitted_picture(xml_content, picture_tag):
    """
    Search picture_tag
    """
    matches = None
    my_regexp = re.compile(r'<{picture_tag}>([^<]+)</{picture_tag}>'.format(picture_tag=picture_tag))
    matches = my_regexp.findall(xml_content)
    return matches


class ManageFile(object):
    """
    Abstract class for XML file read & save
    and data insert/update in XForm & XFormSubmit tables
    """
    request = None
    xml_content = ''
    form_id = ''
    version = ''
    xml_file = ''
    xml_filename = ''
    xform = None

    def __init__(self, request):
        self.request = request

    def process_xformsubmit(self):
        """
        Main function
        return msg, instance
        """
        msg = ''
        instance = None
        msg = self.read_xml(file_key='xml_submission_file')
        if msg == 'OK':
            msg = self.get_key_elements()
            if msg == 'OK':
                # msg = self.get_xform_pk(class_name='XFormSubmit')
                # if msg == 'FOUND':
                msg = self.save_file(file_key='xml_submission_file', class_name='XFormSubmit')
                if msg == 'OK':
                    msg = self.save_pictures()
                    if msg == 'OK':
                        msg, instance = self.insert_in_xformsubmit()                      

        return msg, instance

    def read_xml(self, file_key):
        """
        Get file and read content
        file_key:
        - 'xml_submission_file' for submitted files
        - 'xml_file' for template_files (no more used 2022-06-16)
        """
        if self.request.FILES and file_key in self.request.FILES:
            self.xml_content = self.request.FILES[file_key].read().decode("utf-8")
            # LOG_DEBUG.debug(self.xml_content)
            return 'OK'
        else:
            msg = _("File not found")
            LOG.error = msg
            return msg

    def get_key_elements(self):
        """
        Get key element for processing
        """
        self.form_id = get_form_id(self.xml_content)
        self.version = get_version(self.xml_content)

        if 'ERROR' in [self.form_id, self.version]:
            msg = _("Key elements (form_id - version) not found")
            LOG.error = msg
        else:
            msg = 'OK'
        # print(f"form_id: {self.form_id}")
        # print(f"version: {self.version}")
        return msg

    @property
    def subfolder1(self):
        return f"{self.form_id}-{self.version}"

    @property
    def subfolder2(self):
        survey_date = get_survey_date(self.xml_content)
        if survey_date is None:
            survey_date = datetime.now().strftime("%Y-%m-%d")
        return survey_date


    def get_xform_pk(self, class_name='XForm'):
        """
        Get XForm pk (no more used 2022-06-16)
        """
        try:
            self.xform = XForm.objects.get(form_id=self.form_id, version=self.version)
            msg = 'FOUND'
        except XForm.DoesNotExist:
            if class_name == 'XFormSubmit':
                msg = f"form_id: {self.form_id} - version: {self.version} DoesNotExist in XForm!!!"
                LOG.error(msg)
            else:
                msg = 'NOT_FOUND'
        return msg


    def save_file(self, file_key, class_name):
        """
        Save file on the server
        """
        try:
            file_pointer = self.request.FILES[file_key]
            if class_name == "XFormSubmit":
                self.xml_file = os.path.join(class_name, self.subfolder1, self.subfolder2, file_pointer.name)
            else:
                self.xml_file = os.path.join(class_name, file_pointer.name)
            
            self.xml_filename = file_pointer.name.replace('.xml', '')
            overwrite_storage.save(self.xml_file, file_pointer)
            msg = 'OK'
        except Exception as xcpt:
            LOG.error(xcpt)
            msg = xcpt
        return msg        


    def insert_in_xformsubmit(self):
        """
        Insert submitted data into XFormSubmit table
        Must be last fucntion call!
        """
        try:
            # myxml = xml.dom.minidom.parseString(self.xml_content)
            # xml_pretty_str = myxml.toprettyxml()
            picture_files = get_submitted_picture(self.xml_content, 'pic_img')
            submitted_on = overwrite_storage.get_created_time(self.xml_file)
            instanceid = get_instanceid(self.xml_content)
            try:
                xsub = XFormSubmit.objects.get(instanceid=instanceid)
                # Update
                # xsub.xml_content=xml_pretty_str, # done in models.py
                xsub.picture_files = picture_files,
                xsub.submitted_on = submitted_on
                
            except XFormSubmit.DoesNotExist:
                # Insert
                xsub = XFormSubmit.objects.create(
                    # xform=self.xform,
                    xml_file=self.xml_file,
                    # xml_content=xml_pretty_str, # done in models.py
                    picture_files=picture_files,
                    submitted_on=submitted_on,
                )
            
            pathname = self.xml_file
            lastslash = pathname.rfind('/')
            fname = pathname[lastslash+1:]
            #msg = f'fichier {fname} submitted!'
            msg = _("Well sent! Thanks!")
        except Exception as xcpt:
            LOG.error(xcpt)
            return xcpt, None
        return msg, xsub


    def save_pictures(self):
        """
        Save file on the server
        Only for submitted files
        """
        try:
            picture_files = get_submitted_picture(self.xml_content, f'pic_img')
            for picture_file in picture_files:
                
                if picture_file in self.request.FILES:
                    pic_pointer = self.request.FILES[picture_file]
                    # LOG_DEBUG.debug(f'pic_pointer: {pic_pointer}')
                    pic_path = os.path.join(
                        "XFormSubmit",
                        self.subfolder1,
                        self.subfolder2,
                        self.xml_filename,
                        pic_pointer.name
                    )
                    # LOG_DEBUG.debug(f"pic_path={pic_path}")
                    overwrite_storage.save(pic_path, pic_pointer)
                # else:
                #     LOG_DEBUG.debug('FILE NOT IN request')
                #     LOG_DEBUG.debug(self.request.FILES)
            msg = 'OK'
        except Exception as xcpt:
            LOG.error(xcpt)
            msg = xcpt
        return msg



# Generated by Django 4.0.5 on 2022-06-19 12:53

from django.db import migrations, models
import odk.models


class Migration(migrations.Migration):

    dependencies = [
        ('odk', '0002_remove_xformsubmit_xform'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='xform',
            name='modified_by',
        ),
        migrations.RemoveField(
            model_name='xform',
            name='modified_on',
        ),
        migrations.RemoveField(
            model_name='xformsubmit',
            name='modified_by',
        ),
        migrations.RemoveField(
            model_name='xformsubmit',
            name='modified_on',
        ),
        migrations.AddField(
            model_name='xform',
            name='model_created_on',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Model created on'),
        ),
        migrations.AddField(
            model_name='xformsubmit',
            name='inserted_on',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Inserted on'),
        ),
        migrations.AlterField(
            model_name='xform',
            name='xml_file',
            field=models.FileField(blank=True, help_text="XLSForm converted by <a href='https://getodk.org/xlsform/' target='_blank'>https://getodk.org/xlsform/</a>", null=True, upload_to=odk.models.xform_path, verbose_name='XML file'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
"""

from django.urls import path
from . import views
from odk.openrosa import view_pull, view_push 

__author__ = 'jean pierre huart, patrick houben'
__email__ = 'j.huart@cra.wallonie.be, p.houben@cra.wallonie.be'
__copyright__ = 'Copyright 2021, Jean Pierre Huart'
__license__ = 'GPLv3'
__date__ = '2021-07-20'
__version__ = '0.2 dev'
__status__ = 'Development'


app_name = 'odk'

urlpatterns = [
    path('', views.home, name='home'),
    path('formList/', view_push.rosa_list, name='rosa_list'),
    path('form/<form_id>/', view_push.rosa_detail, name='rosa_detail'),
    path('submission', view_pull.xform_submit, name='rosa_submit'),

    path('list/', views.XFormListView.as_view(), name='xform_list'),
    path('<int:pk>/', views.XFormDetailView.as_view(), name='xform_detail'),
    path('<int:pk>/xls2xml', views.XFormDetailView.as_view(), name='xform_conv2xml'),
    path('<int:pk>/createmodel', views.XFormDetailView.as_view(), name='xform_createmodel'),
    path('add/', views.xform_upload, name='xform_add'),
    path('doc/', views.XFormTemplateView.as_view(), name='xform_doc'),
    # path('<int:pk>/upd', views.xform_upload, name='xform_upd'),
    path('<int:pk>/delete', views.XFormDelView.as_view(), name='xform_del'),

    path('submitted/', views.submittedfile_list, name='xformsubmit_list'),    
    path('submitted/<int:pk>/', views.XFormSubmitDetailView.as_view(), name='xformsubmit_detail'),
    path('submitted/<int:pk>/loaddata/', views.XFormSubmitDetailView.as_view(), name='xformsubmit_load'),
    path('submitted/<int:pk>/delete', views.XFormSubmitDelView.as_view(), name='xformsubmit_del'),    
]



# Generated by Django 3.2.9 on 2021-11-24 09:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('odk', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='xformsubmit',
            name='xform',
        ),
    ]



"""
See PEP 386 (https://www.python.org/dev/peps/pep-0386/)
Release logic:
 1. Increase version number (change __version__ below).
 2. Check that all changes have been documented in CHANGELOG.rst.
 3. git add filer/__init__.py CHANGELOG.rst
 4. git commit -m 'Bump to {new version}'
 5. git push
 6. git tag {new version}
 7. git push --tags
 8. python setup.py sdist bdist_wheel
 9. twine upload dist/django-filer-{new version}.tar.gz
"""

__version__ = "0.3.3"
__author__ = 'Patrick HOUBEN'
__credits__ = 'Walloon Agricultural Research Centre'



# encoding:utf-8
from django import forms
from django.utils.translation import gettext as _
from .models import XForm


class OdkForm(forms.ModelForm):

    class Meta:
        model = XForm
        fields = ['xls_file', 'short_desc']
        widgets = {
            "xls_file": forms.FileInput(attrs={
                "accept": ".xlsx"}
            ),
            "short_desc": forms.TextInput(attrs={
                "class": "form-control"
            })
        }


#!/usr/bin/env python
from setuptools import setup

import odk

with open("README.md", "r") as fh:
    long_description = fh.read()

REQUIREMENTS = [
    'django>=2.2,<4.0',
    'django-braces>=1.11,<2.0',
    'pyxform>=1.10.0,<2.0',
    'xlsconv>=1.3.0,<2.0',
    'pillow>=9.1.1,<9.9',
    'lxml>=4.9.0,<9.0.0',
]

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development',
    'Framework :: Django',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
]

setup(
    name='django-odk',
    version=odk.__version__,
    description='Django Data Collection tool using ODK Collect mobile App',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/openHBP/django-odk',
    author='Patrick HOUBEN',
    author_email='p.houben@cra.wallonie.be',
    license='GPL-3.0-or-later',
    packages=['odk', 'odkdata', 'odk.openrosa'],
    # package_data={'static':['*'], 'templates':['*'], 'templatestags':['*'],},
    include_package_data=True,
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS,
    platforms=['Linux']
)



# -*- coding: utf-8 -*-
"""
odk.admin description
"""
from django.contrib import admin
from django.contrib import messages
from django.utils.translation import gettext as _
from django.utils.timezone import now
from django.shortcuts import render

from .models import XForm, XFormSubmit
from odkdata.utils import rm_digit, convert2camelcase
from odkdata import create_model, load_submit_data


##################
# XForm
##################
@admin.action(description=_('Convert XLSX to XML'))
def xform_convert(modeladmin, request, queryset):
    if queryset.count() > 1:
        messages.warning(request, "Please select record to process 1 by 1", fail_silently=True)
        return
    else:
        record = queryset.first()
        record.process_xform()
        record.save()


@admin.action(description=_('Create Model in odkdata'))
def xform_createmodel(modeladmin, request, queryset):
    """
    TODO
    must use overwrite_storage in admin otherwise filename get HEX extension!
    and odkdata.create_model.xls2django generate bad initial model name
    """
    if queryset.count() > 1:
        messages.warning(request, _("Please select 1 record at a time."), fail_silently=True)
        return
    else:
        record = queryset.first()
        if create_model.process(record):
            messages.success(request, "Model created")
            record.model_created_on = now()
            record.save()
        else:
            messages.error(request, _("Error while creating model."))


class XFormAdmin(admin.ModelAdmin):
    list_display = ('id', 'xls_file', 'form_id', 'version', 'xml_file')
    list_filter = ('xls_file',)
    search_fields = ('xls_file', 'xml_content', 'short_desc', 'title',)
    # fields = ['xls_file', 'xml_content', 'short_desc', 'model_created']
    actions = [xform_convert, xform_createmodel]

    def save_model(self, request, obj, form, change):
        obj.created_by = request.user
        super().save_model(request, obj, form, change)    


##################
# XFormSubmit
##################
def xformsubmit_return_message(return_value, message, request, obj):

    pkg_str = rm_digit(obj.form_id).lower()
    model_str = convert2camelcase(rm_digit(obj.form_id))
    table_name = model_str.lower()

    if return_value == 1:
        load_msg = _("Record loaded with success!")
        messages.success(request, f"{obj} {load_msg}")
        obj.inserted_on = now()
        obj.save()
    elif return_value == -1:
        messages.error(request, message)
        messages.info(request, "Compare 'odkdata.models.{pkg_str}' with 'odkdata.models.{pkg_str}_orig'")                             
        messages.info(request, "odkdata.models.{pkg_str}.{model_str} must have a instanceid field.")
        messages.info(request, "Open an Issue on GitHub repo with xlsx form in a link.")
    elif return_value == -2:
        message = _("table does not exist in the database")
        messages.error(request, f"'odkdata_{table_name}' {message}")
        messages.info(request, _("Did you run 'manage.py makemigrations odkdata' then 'manage.py migrate'?"))
    else:
        # major failure
        messages.error(request, message)
        messages.error(request, f"Error while loading data into 'odkdata.{table_name}', check error logs.")



@admin.action(description=_('Insert in odkdata model'))
def xformsubmit_load_data(modeladmin, request, queryset):
    for obj in queryset:
        return_value, message = load_submit_data.load_record(record)
        xformsubmit_return_message(return_value, message, request, obj)



class XFormSubmitAdmin(admin.ModelAdmin):
    list_display = ('id', 'form_id', 'version', 'xml_file', 'inserted_on', 'picture_files')
    list_filter = ('form_id', 'survey_date')
    search_fields = ('xml_file', 'xml_content', 'title',)
    # fields = ['xml_file', 'picture_files']
    actions = [xformsubmit_load_data]



admin.site.register(XForm, XFormAdmin)
admin.site.register(XFormSubmit, XFormSubmitAdmin)


# -*- coding: utf-8 -*-
import os
import logging

from bs4 import BeautifulSoup

from django.apps import apps
from django.db import connection
from django.contrib.gis.geos.point import Point
from django.db.models import ManyToOneRel, IntegerField
from django.contrib.gis.db.models import PointField

from odk.models import XFormSubmit
from odkdata.utils import rm_digit, convert2camelcase


LOG = logging.getLogger(__name__)



def fetch_xml_data(xfs, model):
    """
    params:
    - XFormSubmit instance
    - model object class

    return: fdict, mdict
    """
    soup = BeautifulSoup(xfs.xml_content, 'xml')

    fdict = {}
    mdict = {}

    for field in model._meta.get_fields():
        value = None
        fname = field.name
                    
        if fname == 'instanceid':
            value = xfs.instanceid
        else:
            if isinstance(field, ManyToOneRel):
                itemtag = soup.find_all(fname)
                value = [i.text for i in itemtag]
                mdict[fname] = value                  
            else:
                itemtag = soup.find(fname)
                if itemtag is not None:
                    value = itemtag.text
                    if isinstance(field, IntegerField):
                        if value == '':
                            value = 0
                        value = int(value)
                    if isinstance(field, PointField):
                        loc_lat, loc_long, loc_alt, precision = value.split(" ")
                        value = Point(x=float(loc_long), y=float(loc_lat), srid=4326)

        # Exclude mdict element for fdict assignment
        if fname not in mdict:
            fdict[fname] = value
    return fdict, mdict  



def load_record(xfs):
    """
    params:
    XFormSubmit instance

    return:
        1 success
        -1 python model not found
        -2 db model not found
        0 failure
    """
    LOG.info(f"-----------------------")
    LOG.info(f"Loading XFormSubmit id: {xfs.id}")

    # Init vars
    form_id = xfs.form_id
    pkg_str = rm_digit(form_id).lower()
    model_str = convert2camelcase(rm_digit(form_id))
    
    # Retrieve model objects class
    try:
        model = apps.get_model("odkdata", model_str)
    except LookupError as xcpt:
        return -1, xcpt

    # Check if table exist in DB
    all_tables = connection.introspection.table_names()
    table_name = f"odkdata_{pkg_str}"
    if table_name not in all_tables:
        return -2, f"{table_name} not in database"

    # Fetch data in XML and return fdict and mdict
    fdict, mdict = fetch_xml_data(xfs, model)

    # Upsert data in main model
    model_inst, created = model.objects.get_or_create(instanceid=xfs.instanceid, defaults=fdict)
    try:
        if not created:
            for attr, value in fdict.items(): 
                if attr not in ('id', 'instanceid'):
                    setattr(model_inst, attr, value)
            model_inst.save()
    except Exception as xcpt:
        LOG.error(xcpt)
        return 0, xcpt


    # Manage ManyToOneRel (mdict)
    keylist = list(mdict.keys())
    model_str_lower = model_str.lower()

    for key in keylist:
        LOG.info(f"Loading ManyToOneRel {key}")
        obj = apps.get_model('odkdata', key.capitalize())
        # first delete all existing occurences
        obj_qs = obj.objects.filter(**{ model_str_lower: model_inst })
        obj_qs.delete()

        multivalues = mdict[key]
        for values in multivalues:
            values = values.strip("\n") # rm leading and trailing \n
            values_list = values.split("\n")
            
            fields = [i.name for i in obj._meta.get_fields() if i.name not in ('id', model_str_lower)]
            n = 0
            objdict = {}
            for f in fields:
                try:
                    value = values_list[n]
                except IndexError:
                    LOG.error(f"fields: {fields}")
                    LOG.error(f"fields number of items: {len(fields)}")
                    LOG.error(f"values_list: {values_list}")
                    LOG.error(f"values_list number of items: {len(values_list)}")
                    return 0, f"IndexError between {fields} and {values_list}"

                objdict[f] = value
                n += 1
            
            # Add instance of main model
            objdict[model_str_lower] = model_inst

            try:
                obj_inst = obj(**objdict)
                obj_inst.save()
            except Exception as xcpt:
                LOG.error(xcpt)
                return 0, xcpt
    
    return 1, "OK"
